local mod = MorgDKP2:NewModule("DKPovertime", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")

local mdkp = MorgDKP2
mod.modName = "DKP/time"
mod.modref = "DKPovertime"

local db
local Player = UnitName("player")
local tinsert = table.insert
local fmt = string.format
local attendees = { }

local defaults = { 
	profile = {
		dkpimmed = nil,
		dkpamount = 1,
		dkpsched = 1800,
		otimename = "<zone> DKP",
		nextaward = nil
	},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return true
end

function mod:GetName()
	return mod.modName, mod.modref
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("DKPovertime", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	
	db.moduleON["DKPovertime"] = true
	self:RestartTimer()
end

function mod:OnDisable()
	db.moduleON["DKPovertime"] = nil
	self:StopTimer()
end

function mod:UpdateDB()
	db = mdkp.db.profile
end

function mod:Toggle(info, v)
	mdkp.db.profile.modules.DKPovertime = v
	if v then mod:Enable()
	else mod:Disable() end
end

function mod:SetOpt(info, v)
	local db = mod.db.profile
	local opt = info[#info]
	if opt == "dkpsched" then 
		db[opt] = floor(v * 60) 
		mod:StopTimer()
		mod:RestartTimer()
	else
		db[opt] = v
	end
end

function mod:GetOpt(info)
	local db = mod.db.profile
	local opt = info[#info]
	if opt == "dkpsched" then return floor(db[opt]/60)
	else return db[opt]
	end
end

function mod:AtRaidStart()
	self:debug("Raidstart DKPovertime")
	if self.db.profile.dkpimmed then
		self.db.profile.nextaward = time()
	else
		self.db.profile.nextaward = time() + self.db.profile.dkpsched
	end
	self:RestartTimer()
end

function mod:AtRaidEnd()
	self:StopTimer()
end

function mod:StopTimer()
	self:CancelTimer(self.timer, true) 
	self.timer = nil
end

function mod:RestartTimer()
	if db.InRaid then
		self:debug("DKP/time event ON")
		local delay = self.db.profile.dkpsched
		local currtime = time()
		if self.db.profile.nextaward and currtime + delay > self.db.profile.nextaward then delay = self.db.profile.nextaward - currtime end
		self.timer = self:ScheduleTimer("RecurrentReward", delay)
	end
end

function mod:RecurrentReward()
	self:debug("DKP/time reward")
	if not db.InRaid then return end 
	
	local POOL = db.raid
	local award = self.db.profile.dkpamount
	local currtime = time()
	local ZoneName = GetRealZoneText()
	attendees = {}
	local counter = 0
	local Runname = db.raidlog[db.raidnum].bosskills[self:GetEventID(2)].note
	local wait = self:GetModuleRef("Waitlist")
	local waitmembers
	if wait and wait.db.profile.waitotime == award then waitmembers = wait:GetWaitlist() end
	
	for name in pairs(mdkp.roster) do
		local testtime = db.raidlog[db.raidnum].join[name]
		if self.db.profile.dkpimmed or currtime - testtime >= self.db.profile.dkpsched then
			tinsert(attendees, name)
			counter = counter + 1
			if not db.dkpevents then
				db.info[name][POOL].points = db.info[name][POOL].points + award
				db.info[name][POOL].earned = db.info[name][POOL].earned + award
			end
		end
	end
	if waitmembers then
		for _, name in pairs(waitmembers) do
			if not self:IsRewarded(name) then
				tinsert(attendees, name)
				counter = counter + 1
			end
		end
	end
	
	if counter > 0 then
		if not db.dkpevents then
			self:AwardWaitotimeDKP(award, POOL, Runname, attendees, "Group", "Group")
			self:out(fmt(L["Added %s DKP to %s attendees."], award, counter))
			if wait and not waitmembers then wait:AwardWaitlistDKPperTime(POOL, currtime, ZoneName, Runname, self.db.profile.dkpsched) end
		else
			local tracker = self:GetModuleRef("Tracker")
			if not self.db.profile.otimename then self.db.profile.otimename = "<zone> DKP" end
			local eventname = self:GetEventName(ZoneName, self.db.profile.otimename)
			if tracker then tracker:LogBossKill(currtime, eventname, true, eventname, nil, 3, attendees, award) end
			if wait and not waitmembers then wait:AwardWaitlistDKPperTimeEvent(POOL, currtime, ZoneName, Runname, self.db.profile.dkpsched) end
			local sync = self:GetModuleRef("Syncing")
			if sync then sync:CustomEvent(eventname) end
		end
		self.db.profile.nextaward = currtime + self.db.profile.dkpsched
	end
	self:RestartTimer()
end

function mod:IsRewarded(name)
	for _, onname in pairs(attendees) do
		if onname == name then return true end
	end
	return false
end
