local mod = MorgDKP2:NewModule("SKall")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")

local mdkp = MorgDKP2

mod.modName = L["SKall DKP"]
mod.modref = "SKall"

local Player = UnitName("player")
local db
local fmt = string.format
local temptable = {}

local defaults = { 
	profile = {
		zeromode = nil,
		skpercent = 100,
	},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return true
end

function mod:GetName()
	return mod.modName, mod.modref
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("SKall", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile	
	db.moduleON["SKall"] = true
	
end

function mod:OnDisable()
	db.moduleON["SKall"] = nil
end

function mod:UpdateDB()
	db = mdkp.db.profile
end

function mod:Toggle(info, v)
	mdkp.db.profile.modules.SKall = v
	if v then mod:Enable()
	else mod:Disable() end
	self:DKPChange(mod.modName)
end

function mod:SetOpt(info, v)
	local db = mod.db.profile
	local opt = info[#info]
	db[opt] = v
end

function mod:GetOpt(info)
	local db = mod.db.profile
	local opt = info[#info]
	return db[opt]
end

function mod:GetPlayerPoints(name, POOL, showearned)
	if not showearned or showearned == 1 then return db.info[name][POOL].points end
	if showearned == 2 then return db.info[name][POOL].earned
	elseif showearned == 3 then return db.info[name][POOL].attend
	end
end

function mod:GetItemValue(itemid, player, POOL)
	local itempoints = self:GetPlayerPoints(player, POOL) * self.db.profile.skpercent / 100
	itempoints = floor(self:GetTAKEModeValue(itemid, itempoints, player) * 100) / 100
	return itempoints
end

function mod:ItemRecorded(id, looter, POOL, points, itempoints, link, attendees, boss)
	if looter == L["Bank"] then return end
	points = points - itempoints
	self:AwardItem(id, looter, POOL, points, itempoints, link)
	
	if self.db.profile.zeromode then
		if not attendees then return end
		temptable = {}
		for _, name in pairs(attendees) do
			tinsert(temptable, name)
		end
	
		--add zero sum points
		if itempoints == 0 then return end
		addpoints = floor(itempoints / #temptable * 100) / 100
		self:AwardDKP(addpoints, 0, POOL, temptable)
	end
end

function mod:ItemRebate(id, looter, POOL, itempoints, link, oldattends, newattends, trashadjust, points)
	if not self.db.profile.zeromode then
		if looter ~= L["Bank"] and id then
			self:AdjustDKPPoints(looter, looter, POOL, nil, itempoints)
		end
	else
		if newattends then		--changing item event
			--remove points from old attendees
			if oldattends then
				local addpoints = floor(itempoints / #oldattends * 100) / 100 
				self:AwardDKP(0, addpoints, POOL, oldattends)
			end
		
			--add points to new attendees
			if looter == L["Bank"] then return end
			local addpoints = floor(itempoints / #newattends * 100) / 100  
			self:AwardDKP(addpoints, 0, POOL, newattends)
		else
			if trashadjust then
				local addpoints = floor(itempoints / #oldattends * 100) / 100 
				points = points - itempoints
				self:AwardItem(id, looter, POOL, points, itempoints, link)
				self:AwardDKP(addpoints, 0, POOL, oldattends)
			elseif oldattends and looter ~= L["Bank"] then
				local addpoints = floor(itempoints / #oldattends * 100) / 100 
				self:AwardDKP(0, addpoints, POOL, oldattends)
				self:AdjustDKPPoints(looter, looter, POOL, nil, itempoints)
			end
		end
	end
end

function mod:ItemAdjustment(id, looter, POOL, newcost, link, oldcost, attends)
	if not self.db.profile.zeromode or not attends or looter == L["Bank"] then return end
	local diff = oldcost - newcost
	local addpoints = floor(diff / #attends * 100) / 100 
	self:AwardDKP(0, addpoints, POOL, attends)
end

function mod:ItemTransfer(id, newmember, oldmember, POOL, newcost, link, oldcost)
	self:AdjustDKPPoints(newmember, oldmember, POOL, newcost, oldcost, link)
end

function mod:IsZeroMode()
	return self.db.profile.zeromode
end
