﻿local MorgDKP2 = LibStub("AceAddon-3.0"):GetAddon("MorgDKP2")
local MorgDKP2_Options = LibStub("AceAddon-3.0"):NewAddon("MorgDKP2_Options")
local C = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2_Options")
local media = LibStub("LibSharedMedia-3.0")
local mdkp = MorgDKP2
local db = MorgDKP2.db.profile
local Core = mdkp:GetModule("CoreModule", true)
local Tracker = mdkp:GetModule("Tracker", true)
local Lootframe = mdkp:GetModule("Lootframe", true)
local PointsDB = mdkp:GetModule("PointsDB", true)

local dkpmodoptions = {}
local ignorelinks = {}
local rankdata = {}
local fmt = string.format

local moduleoptions = {
	["Syncing"] = {
		type = "group",
		handler = mdkp:GetModule("Syncing", true),
		name = L["Syncing"],
		desc = L["Syncing system for communicating DKP between raid leaders."],
		order = 380,
		args = {
			enable = {
				type = "toggle",
				name = L["Enable"],
				desc = L["Syncing system for communicating DKP between raid leaders."],
				get = function() return mdkp.db.profile.modules.Syncing end,
				set = "Toggle",
				order = 5
			},
			auto = {
				type = "toggle",
				name = L["Automatic"],
				desc = L["Automatically set broadcast/receive depending if you are the master looter."],
				get = "GetOpt",
				set = "SetOpt",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 8
			},
			broadcast = {
				type = "toggle",
				name = L["Broadcast"],
				desc = L["Broadcast DKP changes to other raid leaders."],
				get = "GetOpt",
				set = "SetOpt",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 10
			},
			receive = {
				type = "toggle",
				name = L["Receive"],
				desc = L["Accept DKP changes from other raid leaders."],
				get = "GetOpt",
				set = "SetOpt",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 20
			},
			overwrite = {
				type = "toggle",
				name = L["Overwrite raid"],
				desc = L["Overwrite the current raid if you receive an initial sync and are currently in a raid."],
				get = "GetOpt",
				set = "SetOpt",
				hidden = function() return not mdkp.db.profile.modules.Syncing or not mdkp:GetModule("Syncing", true).db.profile.receive end,
				order = 25
			},
			PW = {
				type = "input",
				name = L["Password"],
				desc = L["Syncing password"],
				get = "GetOpt",
				set = 	"SetOpt",
				validate = function(info, v) return v ~= "" end,
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 30
			},
			head1 = {
				type = "header",
				name = L["Immediate Sync Options"],
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 100
			},
			syncdb = {
				type = "execute",
				name = L["Send DB Sync"],
				desc = L["Sends your current database of member DKP values, item values, and MorgDKP2 options to the current raid."],
				func = "SendManualFullSync",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 110
			},
			syncmem = {
				type = "execute",
				name = L["Members Sync"],
				desc = L["Sends your current database of member DKP values."],
				func = "SendManualMemberSync",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 120
			},
			syncitems = {
				type = "execute",
				name = L["Items Sync"],
				desc = L["Sends your current database of item DKP values."],
				func = "SendManualItemSync",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 130
			},
			syncoptions = {
				type = "execute",
				name = L["Options Sync"],
				desc = L["Sends your current MorgDKP2 options.  Note will restart MorgDKP2 so do not send another sync until it restarts or use full sync and it will be handled properly."],
				func = "SendManualOptionsSync",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 140
			},
			resendsync = {
				type = "execute",
				name = L["Initial Sync"],
				desc = L["Resends initial sync.  Only needed in cases where you have problems as this is done automatically."],
				func = "InitialSync",
				hidden = function() return not mdkp.db.profile.modules.Syncing end,
				order = 150
			},
		},
	},
	["BidWar"] = {
		bidwarenable = {
			type = "toggle",
			handler = mdkp:GetModule("BidWar", true),
			name = L["BidWar"],
			desc = L["BidWar mode changes the MorgBid query to allow players to bid on items.  Also the MorgDKP tablet uses bids after this NOT member DKP."],
			get = function() return mdkp.db.profile.modules.BidWar end,
			set = 	"Toggle",
			order = 50
		},
		bidwarhead = {
			type = "header",
			name = L["BidWar Options"],
			hidden = function() return not mdkp.db.profile.modules.BidWar end,
			order = 200
		},
		silent = {
			type = "toggle",
			handler = mdkp:GetModule("BidWar", true),
			name = L["Silent auction"],
			desc = L["Enable BidWar mode with only one round of bidding and no reporting to raid chat."],
			get = "GetOpt",
			set = "SetOpt",
			hidden = function() return not mdkp.db.profile.modules.BidWar end,
			order = 210
		},
		justenough = {
			type = "toggle",
			handler = mdkp:GetModule("BidWar", true),
			name = L["Just enough"],
			desc = L["Enable BidWar mode where the winner is charged the 2nd highest bid value for the item."],
			get = "GetOpt",
			set = "SetOpt",
			hidden = function() return not mdkp.db.profile.modules.BidWar end,
			order = 215
		},
		bidstep = {
			type = "range",
			handler = mdkp:GetModule("BidWar", true),
			name = L["Bidstep"],
			desc = L["Changes the DKP step for BidWar mode.  Passes this value to MorgBid2 clients."],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 100, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.BidWar end,
			order = 220
		},
		overbid = {
			type = "range",
			handler = mdkp:GetModule("BidWar", true),
			name = L["Overbid"],
			desc = L["Maximum amount of DKP that members can exceed their current DKP."],
			get = "GetOpt",
			set = "SetOpt",
			min = 0, 
			max = 500, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.BidWar end,
			order = 225
		},
	},
	["Waitlist"] = {
		type = "group",
		handler = mdkp:GetModule("Waitlist", true),
		name = L["Waitlist"],
		desc = L["Waitlist"],
		order = 400,
		args = {
			enable = {
				type = "toggle",
				name = L["Enable"],
				desc = L["Waitlist allows members to whisper you for invites and to receive DKP even if not in the actual raid."],
				get = function() return mdkp.db.profile.modules.Waitlist end,
				set = "Toggle",
				order = 5
			},
			waitaward = {
				type = "toggle",
				name = L["Award DKP"],
				desc = L["Award DKP to current waitlist members"],
				get = "GetOpt",
				set = "SetOpt",
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 100
			},
			waitrewardtime = {
				type = "range",
				name = L["Waitlist reward"],
				desc = L["Minimum time to award run DKP to a waitlist member."],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 120, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 105
			},
			waitrun = {
				type = "range",
				name = L["Waitlist run DKP"],
				desc = L["Amount of DKP to reward after members have been on the waitlist for the current reward time."],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 110
			},
			waitstart = {
				type = "range",
				name = L["Waitlist start DKP"],
				desc = L["Amount of DKP to reward at raidstart."],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 115
			},
			waitboss = {
				type = "range",
				name = L["Waitlist boss DKP"],
				desc = L["Amount of DKP to reward to waitlist members per bosskill."],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 120
			},
			waitotime = {
				type = "range",
				name = L["Waitlist DKP/time"],
				desc = L["Amount of DKP to reward to waitlist members when awarding regular members DKP/time. Note: only works if you are awarding DKP/time."],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 500, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 120
			},
			waitdeletetime = {
				type = "range",
				name = L["Grace Period"],
				desc = L["Minimum time before offline waitlist members will be removed."],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 120, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 122
			},
			waitaddtime = {
				type = "range",
				name = L["Waitlist add time"],
				desc = L["Maximum time to allow new members to join the waitlist after raidstart. (0 to disable)"],
				get = "GetOpt",
				set = "SetOpt",
				min = 0, 
				max = 120, 
				step = 1,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 123
			},
			head2 = {
				type = "header",
				name = L["Event Names"],
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 125
			},
			waitname = {
				type = "input",
				name = L["Waitlist event name"],
				desc = L["Format of waitlist event names:  \n<zone> text  \nExample:  <zone> Waitlist DKP"],
				get = "GetOpt",
				set = "SetOpt",
				validate = function(info, v) return v ~= "" end,
				hidden = function() return not mdkp.db.profile.modules.Waitlist or not mdkp.db.profile.dkpevents end,
				order = 130
			},
			waitbossname = {
				type = "input",
				name = L["Waitlist boss event name"],
				desc = L["Format of waitlist boss events:  \n<boss> text  \nExample:  <boss> Waitlist DKP"],
				get = "GetOpt",
				set = "SetOpt",
				validate = function(info, v) return v ~= "" end,
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 135
			},
			head3 = {
				type = "header",
				name = L["Waitlist spamming"],
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 140
			},
			dospam = {
				type = "execute",
				name = L["Spam Waitinfo"],
				desc = L["Spam channels with waitinfo now."],
				func = "SpamChannelsDirect",
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 145
			},
			spamchannel = {
				type = "toggle",
				name = L["Autochannel Spam"],
				desc = L["Spam channels automatically when opening the invite tablet."],
				get = "GetOpt",
				set = "SetOpt",
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 150
			},
			channels = {
		 		type = "multiselect",
		 		name = L["Channels"],
		 		desc = L["Channels to spam when starting invites."],
		 		get = "GetOpt",
				set = "SetOpt",
				values = "GetDefaultChannels", 
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 155
			},
			clearwaitlist = {
				type = "execute",
				name = L["Clear Waitlist"],
				desc = L["Clears the current wailist."],
				func = "AtRaidEnd",
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 160
			},
			dospam2 = {
				type = "execute",
				name = L["Spam Current"],
				desc = L["Spam channels with current waitlist."],
				func = "SpamCurrent",
				hidden = function() return not mdkp.db.profile.modules.Waitlist end,
				order = 170
			},
		},
	},
	["DKPovertime"] = {
		timedkpenable = {
			type = "toggle",
			handler = mdkp:GetModule("DKPovertime", true),
			name = L["DKP/time"],
			desc = L["DKP/time system which allows you to award DKP at custom intervals."],
			get = function() return mdkp.db.profile.modules.DKPovertime end,
			set = "Toggle",
			order = 55
		},
		dkptimehead = {
			type = "header",
			name = L["DKP/time Options"],
			hidden = function() return not mdkp.db.profile.modules.DKPovertime end,
			order = 400
		},
		dkpamount = {
			type = "range",
			handler = mdkp:GetModule("DKPovertime", true),
			name = L["Award amount"],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 1000, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.DKPovertime end,
			order = 410
		},
		dkpsched = {
			type = "range",
			handler = mdkp:GetModule("DKPovertime", true),
			name = L["Award time"],
			desc = L["Time in minutes between DKP awards."],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 200, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.DKPovertime end,
			order = 415
		},
		otimename = {
			type = "input",
			handler = mdkp:GetModule("DKPovertime", true),
			name = L["DKP/time event name"],
			desc = L["Format of event name:  \n<zone> text  \nExample:  <zone> DKP"],
			get = "GetOpt",
			set = "SetOpt",
			validate = function(info, v) return v ~= "" end,
			hidden = function() return not mdkp.db.profile.modules.DKPovertime or not mdkp.db.profile.dkpevents end,
			order = 420
		},
		dkpimmed = {
			type = "toggle",
			handler = mdkp:GetModule("DKPovertime", true),
			name = L["Award immediately"],
			desc = L["Awards DKP at raid start and all awards will ignore minimum time in raid."],
			get = "GetOpt",
			set = "SetOpt",
			hidden = function() return not mdkp.db.profile.modules.DKPovertime end,
			order = 430
		},
	},
	["Eventnaming"] = {
		type = "group",
		handler = mdkp:GetModule("Eventnaming", true),
		name = L["Event naming"],
		desc = L["Event naming"],
		order = 300,
		args = {
			enable = {
				type = "toggle",
				name = L["Enable"],
				desc = L["Allows you to change the default format for event names."],
				get = function() return mdkp.db.profile.modules.Eventnaming end,
				set = "Toggle",
				order = 5
			},
			reseteventdefaults = {
				type = "execute",
				name = L["Default"],
				desc = L["Reset default values."],
				func = "ResetDefault",
				hidden = function() return not mdkp.db.profile.modules.Eventnaming end,
				order = 8
			},
			startformat = {
				type = "input",
				name = L["Start format"],
				desc = L["Format of start event name:  \n<zone> text  \nExample:  <zone> <diff> Start"],
				get = "GetOpt",
				set = "SetOpt",
				validate = function(info, v) return v ~= "" end,
				hidden = function() return not mdkp.db.profile.modules.Eventnaming end,
				order = 10
			},
			trashformat = {
				type = "input",
				name = L["Run format"],
				desc = L["Format of run event name:  \n<zone> text  \nExample:  <zone> <diff> Run"],
				get = "GetOpt",
				set = "SetOpt",
				validate = function(info, v) return v ~= "" end,
				hidden = function() return not mdkp.db.profile.modules.Eventnaming end,
				order = 15
			},
			bossformat = {
				type = "input",
				name = L["Boss format"],
				desc = L["Format of boss event names:  \n<zone> - <boss>\nExample:  <boss> <diff>"],
				get = "GetOpt",
				set = "SetOpt",
				validate = function(info, v) return v ~= "" end,
				hidden = function() return not mdkp.db.profile.modules.Eventnaming end,
				order = 20
			},
		},
	},
	["FixedDKP"] = {
		fixeddkp = {
			type = "toggle",
			handler = mdkp:GetModule("FixedDKP", true),
			name = L["Fixed DKP"],
			desc = L["Standard DKP system."],
			get = function() return mdkp.db.profile.modules.FixedDKP end,
			set = "Toggle",
			order = 5
		},	
		head4 = {
			type = "header",
			name = L["DKP TAKE/OFFSPEC settings"],
			hidden = function() return mdkp.db.profile.currDKP == "Random" end,
			order = 300
		},
		takemode = {
			type = "toggle",
			name = L["Enable TAKE"],
			desc = L["Enables mode where members are charged a standard price for TAKE items."],
			get = function() return mdkp.db.profile.takemode end,
			set = function(info, v) mdkp.db.profile.takemode = v end,
			hidden = function() return mdkp.db.profile.currDKP == "Random" end,
			order = 310
		},
		takeamt = {
			type = "range",
			name = L["TAKE value"],
			get = function() return mdkp.db.profile.takeamount end,
			set = 	function(info, v) if v then mdkp.db.profile.takeamount = v end end,
			min = 0, 
			max = 500, 
			step = 1,
			hidden = function() return not mdkp.db.profile.takemode or mdkp.db.profile.currDKP == "Random" end,
			order = 320
		},
		offspectog = {
			type = "toggle",
			name = L["Enable OFFSPEC"],
			desc = L["Enables mode where members are charged a standard price for OFFSPEC items."],
			get = function() return mdkp.db.profile.offspecmode end,
			set = function(info, v) mdkp.db.profile.offspecmode = v end,
			hidden = function() return mdkp.db.profile.currDKP == "Random" end,
			order = 330
		},
		offspecamt = {
			type = "range",
			name = L["OFFSPEC value"],
			get = function() return mdkp.db.profile.offspecamount end,
			set = 	function(info, v) if v then mdkp.db.profile.offspecamount = v end end,
			min = 0, 
			max = 500, 
			step = 1,
			hidden = function() return not mdkp.db.profile.offspecmode or mdkp.db.profile.currDKP == "Random" end,
			order = 340
		},	
		usetakeperc = {
			type = "toggle",
			name = L["TAKE Percent"],
			desc = L["Use percent of item value for TAKE mode."],
			get = function() return mdkp.db.profile.takepercent end,
			set = function(info, v) mdkp.db.profile.takepercent = v end,
			hidden = function() return not mdkp.db.profile.takemode or mdkp.db.profile.currDKP == "Random" end,
			order = 350
		},
		useoffperc = {
			type = "toggle",
			name = L["OFFSPEC Percent"],
			desc = L["Use percent of item value for OFFSPEC mode."],
			get = function() return mdkp.db.profile.offspecpercent end,
			set = function(info, v) mdkp.db.profile.offspecpercent = v end,
			hidden = function() return not mdkp.db.profile.offspecmode or mdkp.db.profile.currDKP == "Random" end,
			order = 350
		},
	},
	["Percent"] = {
		percentdkp = {
			type = "toggle",
			handler = mdkp:GetModule("Percent", true),
			name = L["Percent DKP"],
			desc = L["Items will cost a percentage of the members total DKP."],
			get = function() return mdkp.db.profile.modules.Percent end,
			set = "Toggle",
			order = 35
		},
		percenthead = {
			type = "header",
			name = L["Percent DKP Options"],
			hidden = function() return not mdkp.db.profile.modules.Percent end,
			order = 100
		},
		percent = {
			type = "range",
			handler = mdkp:GetModule("Percent", true),
			name = L["Percent cost"],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 100, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.Percent end,
			order = 110
		},
		percenttake = {
			type = "toggle",
			handler = mdkp:GetModule("Percent", true),
			name = L["Percent take"],
			desc = L["When using the percent DKP system this option enables TAKE looting.  This means if the winning member rolled TAKE he will not be charged x% DKP he will instead be charged the base price of the item (Set using original itempoints."],
			get = "GetOpt",
			set = "SetOpt",
			hidden = function() return not mdkp.db.profile.modules.Percent end,
			order = 120
		},
	},
	["Random"] = {
		randomdkp = {
			type = "toggle",
			handler = mdkp:GetModule("Random", true),
			name = L["Random rolls"],
			desc = L["Enable random rolling for items."],
			get = function() return mdkp.db.profile.modules.Random end,
			set = "Toggle",
			order = 15
		},
		randomhead = {
			type = "header",
			name = L["Random Options"],
			hidden = function() return not mdkp.db.profile.modules.Random end,
			order = 100
		},
		weightedrandom = {
			type = "toggle",
			name = L["Weighted Random"],
			desc = L["Use attendance percent as a multiplier for random rolls. (roll * att/100)"],
			get = function() return mdkp.db.profile.weighrand end,
			set = function(info, v) mdkp.db.profile.weighrand = v end,
			hidden = function() return not mdkp.db.profile.modules.Random end,
			order = 110
		},
	},
	["Relational"] = {
		relationaldkp = {
			type = "toggle",
			handler = mdkp:GetModule("Relational", true),
			name = L["Relational DKP"],
			desc = L["Enable Relational DKP for calculating the earned/spent dkp ratio."],
			get = function() return mdkp.db.profile.modules.Relational end,
			set = "Toggle",
			order = 10
		},
		relationalhead = {
			type = "header",
			name = L["Relational DKP Options"],
			hidden = function() return not mdkp.db.profile.modules.Relational end,
			order = 100
		},
		minDKP = {
			type = "range",
			handler = mdkp:GetModule("Relational", true),
			name = L["Minimum GP"],
			desc = L["Minimum amount of GP a member can have for calculating the DKP ratio to avoid a new member having a high ratio. ie  200 earned DKP/ (this value)"],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 1000, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.Relational end,
			order = 110
		},
 		baseDKP = {
 			type = "range",
 			handler = mdkp:GetModule("Relational", true),
			name = L["BaseGP"],
 			desc = L["GP all members have, added to what they've actually spent on gear. I.e.  EP/(GP + BaseGP)"],
 			get = "GetOpt",
			set = "SetOpt",
			min = 0, 
 			max = 1000, 
 			step = 1,
 			hidden = function() return not mdkp.db.profile.modules.Relational end,
 			order = 120
 		},
 	},
 	["SKall"] = {
 		skalldkp = {
			type = "toggle",
			handler = mdkp:GetModule("SKall", true),
			name = L["SKall DKP"],
			desc = L["Enable Suicide Kings spend all DKP."],
			get = function() return mdkp.db.profile.modules.SKall end,
			set = "Toggle",
			order = 20
		},
		skhead = {
			type = "header",
			name = L["SKall Options"],
			hidden = function() return not mdkp.db.profile.modules.SKall end,
			order = 100
		},
		zeromode = {
			type = "toggle",
			handler = mdkp:GetModule("SKall", true),
			name = L["Enable raid award"],
			desc = L["Enables DKP removed from item winner to be awarded to raid members.  You must check use zerosum on the import webscript for website DKP to be correct."],
			get = "GetOpt",
			set = "SetOpt",
			hidden = function() return not mdkp.db.profile.modules.SKall end,
			order = 110
		},
		skpercent = {
			type = "range",
			handler = mdkp:GetModule("SKall", true),
			name = L["% Member DKP"],
			desc = L["Percent of member DKP to remove."],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 100, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.SKall end,
			order = 120
		},
	},
	["SKSotC"] = {
		skallSOTCdkp = {
			type = "toggle",
			handler = mdkp:GetModule("SKSotC", true),
			name = L["SKSotC DKP"],
			desc = L["Enable Suicide Kings SotC variant DKP."],
			get = function() return mdkp.db.profile.modules.SKSotC end,
			set = "Toggle",
			order = 20
		},
		sksotchead = {
			type = "header",
			name = L["SKSotC Options"],
			hidden = function() return not mdkp.db.profile.modules.SKSotC end,
			order = 100
		},
		sksotchead2 = {
			type = "description",
			name = "",
			hidden = function() return not mdkp.db.profile.modules.SKSotC end,
			order = 102
		},
		demoevent = {
			type = "input",
			handler = mdkp:GetModule("SKSotC", true),
			name = L["Demotion event"],
			desc = L["Format of event name:  \n<zone> text  \nExample:  <zone> DKP"],
			get = "GetOpt",
			set = "SetOpt",
			validate = function(info, v) return v ~= "" end,
			hidden = function() return not mdkp.db.profile.modules.SKSotC end,
			order = 105
		},
		demodkp = {
			type = "range",
			handler = mdkp:GetModule("SKSotC", true),
			name = L["Missed raid DKP"],
			desc = L["Amount of DKP to remove from members who miss a raid."],
			get = "GetOpt",
			set = "SetOpt",
			min = 0, 
			max = 100, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.SKSotC end,
			order = 110
		},
	},
	["TakeBid"] = {
		takebiddkp = {
			type = "toggle",
			handler = mdkp:GetModule("TakeBid", true),
			name = L["TakeBid DKP"],
			desc = L["Variation of Fixed DKP where when you bid take you also enter what you are willing to pay."],
			get = function() return mdkp.db.profile.modules.TakeBid end,
			set = "Toggle",
			order = 30
		},
		takebidhead = {
			type = "header",
			name = L["TakeBid Options"],
			hidden = function() return not mdkp.db.profile.modules.TakeBid end,
			order = 100
		},
		takebidstep = {
			type = "range",
			handler = mdkp:GetModule("TakeBid", true),
			name = L["Bidstep"],
			desc = L["Changes the DKP step for TakeBid mode.  Passes this value to MorgBid2 clients."],
			get = "GetOpt",
			set = "SetOpt",
			min = 1, 
			max = 100, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.TakeBid end,
			order = 110
		},
		takeoverbid = {
			type = "range",
			handler = mdkp:GetModule("TakeBid", true),
			name = L["Overbid"],
			desc = L["Maximum amount of DKP that members can exceed their current DKP."],
			get = "GetOpt",
			set = "SetOpt",
			min = 0, 
			max = 500, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.TakeBid end,
			order = 120
		},
		takeminbid = {
			type = "range",
			handler = mdkp:GetModule("TakeBid", true),
			name = L["Minimum bid"],
			desc = L["Minimum amount of DKP that members can bid for an item."],
			get = "GetOpt",
			set = "SetOpt",
			min = 0, 
			max = 100, 
			step = 1,
			hidden = function() return not mdkp.db.profile.modules.TakeBid end,
			order = 125
		},
	},
	["Zerosum"] = {
		zerodkp = {
			type = "toggle",
			handler = mdkp:GetModule("Zerosum", true),
			name = L["Zerosum DKP"],
			desc = L["Standard zero sum system where member is charged for the item and the charged points are divided between all raid members.  You must check use zerosum on the import webscript for website DKP to be correct."],
			get = function() return mdkp.db.profile.modules.Zerosum end,
			set = "Toggle",
			order = 40
		},
		zerohead = {
			type = "header",
			name = L["Zerosum Options"],
			hidden = function() return not mdkp.db.profile.modules.Zerosum end,
			order = 100
		},
		usewait = {
			type = "toggle",
			handler = mdkp:GetModule("Zerosum", true),
			name = L["Include waitlist"],
			desc = L["Include waitlist members in DKP calculations.  Note they will still get whatever DKP award is set in the waitlist module."],
			get = "GetOpt",
			set = "SetOpt",
			hidden = function() return not mdkp.db.profile.modules.Zerosum end,
			order = 110
		},
	},
}

local options = {
	name = "MorgDKP2",
	handler = mdkp,
	type = "group",
	args = { 
		toggle = {
			type = "toggle",
			name = L["Toggle MorgDKP2"],
			desc = L["Toggle MorgDKP2"],
			get = function() return mdkp:GetCoreState() end,
			set = 	function(info, v) 
					if v then mdkp:ActivateCore(true) 
					else mdkp:DeActivateCore(true) end
				end,
			order = 5
		},
		autozone = {
			type = "toggle",
			name = L["Autozone"],
			desc = L["Automatically enable MorgDKP2 and start a raid when entering a trackable zone."],
			get = function() return mdkp.db.profile.autozone end,
			set = function(info, v) mdkp.db.profile.autozone = v end,
			disabled = function() return not mdkp:IsEnabled() end,
			order = 10
		},
		devmode = {
			type = "toggle",
			name = L["Developer mode"],
			desc = L["Developer mode"],
			get = function() return mdkp.db.profile.devmode end,
			set = function(info, v) mdkp.db.profile.devmode = v end,
			disabled = function() return not mdkp:IsEnabled() end,
			order = 15
		},
		clearraiddb = {
			type = "execute",
			name = L["Clear raid DB"],
			desc = L["Clear currently tracked raids."],
			func = function() mdkp:ClearRaidDB() end,
			disabled = function() return not mdkp:IsEnabled() end,
			order = 20
		},
		modules = {
			type = "group",
			name = L["Core Modules"],
			desc = L["Core Modules"],
			order = 200,
			args = {
				coremod = {
					type = "group",
					handler = MorgDKP2_Options,
					name = L["Core Options"],
					desc = L["Core of all MorgDKP2 features."],
					order = 5,
					args = {
						mlde = {
							type = "toggle",
							name = L["Enable ML/DE mode"],
							desc = L["If ML/DE Mode is on then no items will be recorded for the master looter or disenchanter defined by MorgDKP2."],
							get = function() return mdkp.MLmode end,
							set = function(info, v) mdkp.MLmode = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 5
						},
						mbidc = {
							type = "execute",
							name = L["Morgbid check"],
							desc = L["Determine MorgBid user base. \nAlso useful to check DKP for alts."],
							func = function() Core:MorgBidQuery() end,
							disabled = function() return not Core:IsEnabled() end,
							order = 10
						},
						hints = {
							type = "toggle",
							name = L["Enable tooltip hints"],
							desc = L["Enable tooltip hints"],
							get = function() return mdkp.db.profile.hints end,
							set = function(info, v) mdkp.db.profile.hints = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 15
						},
						toggleminimap = {
							type = "toggle",
							name = L["Show minimap icon"],
							desc = L["Show minimap icon"],
							get = function() 
								local ldbmod = mdkp:GetModule("LDBPlugin", true)
								if ldbmod then return not ldbmod.db.profile.minimapicon.hide end
							end,
							set = function(info, v) 
									local ldbmod = mdkp:GetModule("LDBPlugin", true)
									if ldbmod then
										ldbmod.db.profile.minimapicon.hide = not v
										ldbmod:ToggleMinimapIcon(v)
									end
							end,
							disabled = function() return not Core:IsEnabled() end,
							order = 16
						},
						whisper = {
							type = "toggle",
							name = L["Whisper system"],
							desc = L["Whispers all members (even if they have MorgBid2) so they can reply and bid using whispers."],
							get = function() return mdkp.db.profile.modules.Whisper end,
							set = 	function(info, v) 
									mdkp.db.profile.modules.Whisper = v
									local whisper = mdkp:GetModule("Whisper", true)
									if whisper then 
										if v then whisper:Enable()
										else whisper:Disable() end
									end
								end,
							disabled = function() return not Core:IsEnabled() end,
							order = 18
						},
						linkraid = {
							type = "toggle",
							name = L["Link to Raid"],
							desc = L["Link items on the boss/mob that are above the loot threshold to raid chat."],
							get = function() return mdkp.db.profile.raidlink end,
							set = function(info, v) mdkp.db.profile.raidlink = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 20
						},
						linkguild = {
							type = "toggle",
							name = L["Link to Guild"],
							desc = L["Link items on the boss/mob that are above the loot threshold to guild chat."],
							get = function() return mdkp.db.profile.guildlink end,
							set = function(info, v) mdkp.db.profile.guildlink = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 25
						},
						showearned = {
							type = "select",
							name = L["Invite DKP format"],
							desc = L["Use DKP balance/earned DKP/attendance in the invite tablet (shift-click)."],
							get = function() return mdkp.db.profile.showearned end,
							set = function(info, v) mdkp.db.profile.showearned = v end,
							values = mdkp.Inviteformat,
							order = 27
						},
						ignore58 = {
							type = "toggle",
							name = L["Ignore 6-8"],
							desc = L["Ignore members in groups 6-8 when querying for items."],
							get = function() return mdkp.db.profile.ignore68 end,
							set = function(info, v) mdkp.db.profile.ignore68 = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 30
						},
						ignoreunique = {
							type = "toggle",
							name = L["Ignore worn"],
							desc = L["Ignore items worn by members.  This means queries will always be sent even if they have the item."],
							get = function() return mdkp.db.profile.ignoreunique end,
							set = function(info, v) mdkp.db.profile.ignoreunique = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 35
						},
						saveilvl = {
							type = "range",
							name = L["ILvl"],
							desc = L["Only items above this iLvl will be saved in the players equipment DB."],
							get = function() return mdkp.db.profile.savelvl end,
							set = 	function(info, v) mdkp.db.profile.savelvl = v end,
							min = 2, 
							max = 300, 
							step = 1,
							disabled = function() return not Core:IsEnabled() end,
							order = 40
						},
						display = {
							type = "header",
							name = L["Display Options"],
							desc = L["General display options for main modules."],
							order = 100
						},
						font = {
					 		type = "select",
					 		dialogControl = "LSM30_Font",
					 		name = L["Font"],
					 		desc = L["Font used in tooltips and frames."],
					 		get = function() return mdkp.db.profile.media.font end,
					 		set = function(info, v) mdkp.db.profile.media.font = v MorgDKP2_Options:UpdateFrameMedia() end,
					 		values = AceGUIWidgetLSMlists.font, 
							disabled = function() return not Core:IsEnabled() end,
							order = 120
						},
						fontsize = {
							type = "range",
							name = L["Font size"],
							get = function() return mdkp.db.profile.media.fontsize end,
							set = function(info, v) mdkp.db.profile.media.fontsize = v MorgDKP2_Options:UpdateFrameMedia() end,
							min = 1, 
							max = 30, 
							step = 1,
							disabled = function() return not Core:IsEnabled() end,
							order = 125
						},
						background = {
							type = "select",
							dialogControl = "LSM30_Background",
							name = L["Background texture"],
							desc = L["Background texture of frames/tooltips."],
							get = function() return mdkp.db.profile.media.background end,
							set = function(info, v) 
									mdkp.db.profile.media.background = v
									MorgDKP2_Options:UpdateFrameMedia()
								end,
							values = AceGUIWidgetLSMlists.background, 
							disabled = function() return not Core:IsEnabled() end,
							order = 130
						},
						border = {
							type = "select",
							dialogControl = "LSM30_Border",
							name = L["Border texture"],
							desc = L["Border texture of frames/tooltips."],
							get = function() return mdkp.db.profile.media.border end,
							set = function(info, v) 
									mdkp.db.profile.media.border = v
									MorgDKP2_Options:UpdateFrameMedia()
								end,
							values = AceGUIWidgetLSMlists.border, 
							disabled = function() return not Core:IsEnabled() end,
							order = 135
						},
						scale = {
							type = "range",
							name = L["Frame scale"],
							get = function() return mdkp.db.profile.media.scale end,
							set = function(info, v) mdkp.db.profile.media.scale = v MorgDKP2_Options:UpdateFrameMedia() end,
							min = .5, 
							max = 2, 
							step = .1,
							disabled = function() return not Core:IsEnabled() end,
							order = 137
						},
						desc = {
							type = "description",
							name = "",
							order = 138
						},
						backdropC = {
							type = "color",
							name = L["Background color"],
							hasAlpha = true,
							set = "SetColorOpt",
							get = "GetColorOpt",
							disabled = function() return not Core:IsEnabled() end,
							order = 140
						},
						borderC = {
							type = "color",
							name = L["Border color"],
							hasAlpha = true,
							set = "SetColorOpt",
							get = "GetColorOpt",
							disabled = function() return not Core:IsEnabled() end,
							order = 145
						},
						fontC = {
							type = "color",
							name = L["Font color"],
							hasAlpha = true,
							set = "SetColorOpt",
							get = "GetColorOpt",
							disabled = function() return not Core:IsEnabled() end,
							order = 150
						},
						head3 = {
							type = "header",
							name = L["MorgBid2 Options"],
							order = 200
						},
						silent = {
							type = "toggle",
							name = L["MorgBid2 updating"],
							desc = L["Allow updating bid results in MorgBid2."],
							get = function() return mdkp.db.profile.mbidupdating end,
							set = function(info, v) mdkp.db.profile.mbidupdating = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 210
						},
						mbidnag = {
							type = "toggle",
							name = L["MorgBid2 Nag"],
							desc = L["Enables nagging outdated MorgBid2 users."],
							get = function() return mdkp.db.profile.mbidnag end,
							set = function(info, v) mdkp.db.profile.mbidnag = v end,
							disabled = function() return not Core:IsEnabled() end,
							order = 211
						},
						mbidcustom = {
							type = "toggle",
							name = L["Custom MorgBid2"],
							desc = L["Enables changing the text of button hints in MorgBid2."],
							get = function() return mdkp.db.profile.mbidcustom end,
							set = function(info, v) 
									mdkp.db.profile.mbidcustom = v 
									if v then Core:CreateCustomMbidTable() 
									else Core:RestoreDefaultMbidTable() end
							end,
							disabled = function() return not Core:IsEnabled() end,
							order = 212
						},
						useoffspec = {
							type = "toggle",
							name = L["Use OFFSPEC"],
							desc = L["Allow the OFFSPEC button to be used in MorgBid2."],
							get = function() return mdkp.db.profile.mbidoffspec end,
							set = function(info, v) 
									mdkp.db.profile.mbidoffspec = v 
									Core:CreateCustomMbidTable() 
							end,
							disabled = function() return not Core:IsEnabled() end,
							hidden = function() return not mdkp.db.profile.mbidcustom end,
							order = 215
						},
						desc2 = {
							type = "description",
							name = "",
							hidden = function() return not mdkp.db.profile.mbidcustom end,
							order = 230
						},
						need = {
							type = "input",
							name = L["NEED text"],
							desc = L["Text of MorgBid2 NEED button"],
							get = function() return mdkp.db.profile.NEED end,
							set = 	function(info, v) 
									local text = Core:CheckFormat(v)
									if text then mdkp.db.profile.NEED = text end
									Core:CreateCustomMbidTable() 
							end,
							validate = function(info, v) return v ~= "" end,
							disabled = function() return not Core:IsEnabled() end,
							hidden = function() return not mdkp.db.profile.mbidcustom end,
							order = 220
						},
						take = {
							type = "input",
							name = L["TAKE text"],
							desc = L["Text of MorgBid2 TAKE button"],
							get = function() return mdkp.db.profile.TAKE end,
							set = 	function(info, v) 
									local text = Core:CheckFormat(v)
									if text then mdkp.db.profile.TAKE = text end
									Core:CreateCustomMbidTable() 
							end,
							validate = function(info, v) return v ~= "" end,
							disabled = function() return not Core:IsEnabled() end,
							hidden = function() return not mdkp.db.profile.mbidcustom end,
							order = 230
						},
						offspec = {
							type = "input",
							name = L["OFFSPEC text"],
							desc = L["Text of MorgBid2 OFFSPEC button"],
							get = function() return mdkp.db.profile.OFFSPEC end,
							set = 	function(info, v) 
									local text = Core:CheckFormat(v)
									if text then mdkp.db.profile.OFFSPEC = text end
									Core:CreateCustomMbidTable() 
							end,
							validate = function(info, v) return v ~= "" end,
							disabled = function() return not Core:IsEnabled() end,
							hidden = function() return not mdkp.db.profile.mbidcustom or not mdkp.db.profile.mbidoffspec end,
							order = 240
						},
						pass = {
							type = "input",
							name = L["PASS text"],
							desc = L["Text of MorgBid2 PASS button"],
							get = function() return mdkp.db.profile.PASS end,
							set = 	function(info, v) 
									local text = Core:CheckFormat(v)
									if text then mdkp.db.profile.PASS = text end
									Core:CreateCustomMbidTable() 
							end,
							validate = function(info, v) return v ~= "" end,
							disabled = function() return not Core:IsEnabled() end,
							hidden = function() return not mdkp.db.profile.mbidcustom end,
							order = 250
						},
			        	},
			        },
			        tracker = {
			        	type = "group",
					handler = Tracker,
					name = L["DKP Tracking"],
					desc = L["Core of all MorgDKP2 raid tracking features. \nCan not be disabled."],
					order = 100,
					args = {
						head = {
							type = "header",
							name = L["General options"],
							order = 10
						},
						usestart = {
							type = "toggle",
							name = L["Start Event"],
							desc = L["Use start event when raid begins."],
							get = function() return mdkp.db.profile.startrun end,
							set = function(info, v) mdkp.db.profile.startrun = v end,
							order = 100
						},	
						onerun = {
							type = "toggle",
							name = L["Single run event"],
							desc = L["Use one event for entire raid.  Boss kills will not be tracked separately."],
							get = function() return mdkp.db.profile.OneEvent end,
							set = function(info, v) mdkp.db.profile.OneEvent = v end,
							order = 105
						},
						eqdkpplus = {
							type = "toggle",
							name = L["Eqdkp+ event format"],
							desc = L["Remove the ' character from any event names because eqdkp+ is dumb."],
							get = function() return mdkp.db.profile.eqdkp end,
							set = function(info, v) mdkp.db.profile.eqdkp = v end,
							order = 110
						},
						dkplisten = {
							type = "toggle",
							name = L["DKP listener"],
							desc = L["When you are in a tracked zone you will track DKP but you will not get the loot popup window or link to the raid.  You will act like a backup raidlog."],
							get = function() return mdkp.db.profile.dkplistener end,
							set = function(info, v) mdkp.db.profile.dkplistener = v end,
							order = 115
						},
						ctrtexport = {
							type = "select",
							name = L["Export format"],
							desc = L["Format to use for raid exports."],
							get = function() return mdkp.db.profile.exportformat end,
							set = function(info, v) mdkp.db.profile.exportformat = v end,
							values = mdkp.Exports,
							order = 116
						},
						useevents = {
							type = "toggle",
							name = L["Events for DKP changes"],
							desc = L["Use events for DKP changes instead of database adjustments."],
							get = function() return mdkp.db.profile.dkpevents end,
							set = function(info, v) mdkp.db.profile.dkpevents = v end,
							order = 118
						},
						bank = {
							type = "toggle",
							name = L["Bank character"],
							desc = L["Use a bank character in raids."],
							get = function() return mdkp.db.profile.bank end,
							set = function(info, v) mdkp.db.profile.bank = v end,
							order = 120
						},
						killtimes = {
							type = "toggle",
							name = L["Remember kill times"],
							desc = L["Store your best kill times for bosses."],
							get = function() return Tracker.db.profile.savekilltimes end,
							set = function(info, v) Tracker.db.profile.savekilltimes = v end,
							order = 121
						},
						head2 = {
							type = "description",
							name = "",
							order = 122
						},
						rewardtime = {
							type = "range",
							name = L["Attendance reward"],
							desc = L["Minimum time to add member to raid and award run DKP."],
							get = function() return floor(Tracker.db.profile.runrewardtime / 60) end,
							set = function(info, v) 
									Tracker.db.profile.runrewardtime = floor(v * 60) 
									Tracker:StopAttEvent()
									Tracker:StartAttEvent()
							end,
							min = 0, 
							max = 120, 
							step = 1,
							order = 123
						},
						needattendcutoff = {
							type = "range",
							name = L["Attendance based NEED"],
							desc = L["Minimum attendance percentage to allow MorgBid2 clients to roll NEED. 0 is disabled."],
							get = function() return mdkp.db.profile.attendneed end,
							set = function(info, v) mdkp.db.profile.attendneed  = v end,
							min = 0, 
							max = 100, 
							step = 1,
							order = 124
						},
						newdkp = {
							type = "range",
							name = L["New member start DKP"],
							desc = L["DKP which new members to the database receive."],
							get = function() return mdkp.db.profile.newdkp end,
							set = function(info, v) mdkp.db.profile.newdkp = v end,
							min = 0, 
							max = 200, 
							step = 1,
							order = 125
						},
						newatt = {
							type = "range",
							name = L["New member initial attendance"],
							desc = L["Attendance value(%) for new members to the database."],
							get = function() return mdkp.db.profile.newatt end,
							set = function(info, v) mdkp.db.profile.newatt = v end,
							min = 0, 
							max = 100, 
							step = 10,
							order = 127
						},
						newdkpname = {
							type = "input",
							name = L["New member DKP event"],
							desc = L["Format of new member DKP event name."],
							get = function() return mdkp.db.profile.newdkpname end,
							set = 	function(info, v) if v then mdkp.db.profile.newdkpname = v end end,
							validate = function(info, v) return v ~= "" end,
							hidden = function() return mdkp.db.profile.newdkp == 0 end,
							order = 130
						},
						head1 = {
							type = "header",
							name = L["Shortcuts"],
							order = 200
						},
						startrd = {
							type = "execute",
							name = L["Start Raid"],
							desc = L["Starts a new raid."],
							func = function() Tracker:BeginRaid() end,
							disabled = function() return not Core:IsEnabled() or mdkp.db.profile.InRaid end,
							order = 201
						},
						endrd = {
							type = "execute",
							name = L["End Raid"],
							desc = L["Finalizes the current raid."],
							func = function() Tracker:ShowEndRaidFrame() end,
							disabled = function() return not Core:IsEnabled() or not mdkp.db.profile.InRaid end,
							order = 202
						},
						runbonus = {
							type = "execute",
							name = L["Export Raids"],
							desc = L["Exports current raid(s) data for import into website."],
							func = function() Tracker:ExportRaids() end,
							disabled = function() return not Core:IsEnabled() end,
							order = 203
						},
						raidtracker = {
							type = "execute",
							name = L["Raid Tracker"],
							desc = L["Opens the raidtracker interface."],
							func = function() 
									local raidtracker = mdkp:GetModule("RaidTracker", true)
									if raidtracker then raidtracker:OpenFrame() end
							 end,
							disabled = function() return not Core:IsEnabled() or not mdkp.db.profile.moduleON.RaidTracker end,
							order = 205
						},
						
					},
				},
				lootframe = {
					type = "group",
					handler = Lootframe,
					name = L["Lootframe"],
					desc = L["Lootframe"],
					order = 340,
					args = {
						replaceblizz = {
							type = "toggle",
							name = L["Replace Blizzard"],
							desc = L["When MorgDKP2 is active it's loot frame will replace default."],
							get = function() return Lootframe.db.profile.usemorg end,
							set = 	function(info, v) 
									Lootframe.db.profile.usemorg = v
									if v then Lootframe:EnableBlizzHooks()
									else Lootframe:DisableBlizzHooks() end
								end,
							order = 1
						},
						atcursor = {
							type = "toggle",
							name = L["Snap to cursor"],
							desc = L["Open lootframe at cursor postition"],
							get = function() return Lootframe.db.profile.cursor end,
							set = 	function(info, v) Lootframe.db.profile.cursor = v end,
							order = 5
						},
						useinactive = {
							type = "toggle",
							name = L["Always use"],
							desc = L["Always use the MorgDKP2 loot window even when the mod is in standby mode."],
							get = function() return mdkp.db.profile.alwaysloot end,
							set = 	function(info, v) mdkp.db.profile.alwaysloot = v end,
							hidden = function() return not Lootframe.db.profile.usemorg end,
							order = 3
						},
						desc = {
							type = "description",
							name = "",
							order = 10
						},
						lootscale = {
							type = "range",
							name = L["Lootframe scale"],
							get = function() return Lootframe.db.profile.scale end,
							set = function(info, v) if v then Lootframe.db.profile.scale = v end Lootframe:UpdateLootFrameMedia() end,
							min = .5, 
							max = 2, 
							step = .1,
							order = 15
						},
				      	},
				},
				pointsdb = {
					type = "group",
					handler = MorgDKP2_Options,
					name = L["Database Functions"],
					desc = L["Database Functions"],
					order = 200,
					args = {
						clearraiddb = {
							type = "execute",
							name = L["Clear raid DB"],
							desc = L["Clears the current tracked raids."],
							func = "ClearRaidDB",
							disabled = function() return not Core:IsEnabled() end,
							order = 1
						},
						clear = {
							type = "execute",
							name = L["Clear player DB"],
							desc = L["Clears all players currently in the database. Does not affect items."],
							func = "ClearDatabase",
							disabled = function() return not Core:IsEnabled() end,
							order = 3
						},
						zeroplayerdb = {
							type = "execute",
							name = L["Zero player DB"],
							desc = L["Resets all members DKP to zero."],
							func = "ZeroPLayerDB",
							disabled = function() return not Core:IsEnabled() end,
							order = 4
						},
						head = {
							type = "header",
							name = "",
							order = 6
						},
						additem = {
							type = "input",
							name = L["Add item"],
							desc = L["Add new item to track.  Drag the item to this input box."],
							set = 	"AddNewItem",
							disabled = function() return not Core:IsEnabled() end,
							order = 8
						},
						desc = {
							type = "description",
							name = "",
							order = 9
						},
						cullllvl = {
							type = "range",
							name = L["Cull item level"],
							get = function() return PointsDB.db.profile.culllvl end,
							set = 	function(info, v) PointsDB.db.profile.culllvl = v end,
							min = 2, 
							max = 300, 
							step = 1,
							disabled = function() return not Core:IsEnabled() end,
							order = 10
						},
						cullitemdb = {
							type = "execute",
							name = L["Cull item DB"],
							desc = L["Clears all items in the database below the selected item level."],
							func = "CullItemDatabase",
							disabled = function() return not Core:IsEnabled() end,
							order = 15
						},
						desc2 = {
							type = "description",
							name = "",
							order = 16
						},
						cullmemllvl = {
							type = "range",
							name = L["Cull member level (weeks)"],
							get = function() return PointsDB.db.profile.memculllvl end,
							set = 	function(info, v) PointsDB.db.profile.memculllvl = v end,
							min = 1, 
							max = 16, 
							step = 1,
							disabled = function() return not Core:IsEnabled() end,
							order = 17
						},
						cullmemberdb = {
							type = "execute",
							name = L["Cull member DB"],
							desc = L["Clears all members in the database inactive longer than the selected number of weeks."],
							func = "CullMemberDatabase",
							disabled = function() return not Core:IsEnabled() end,
							order = 18
						},
						head2 = {
							type = "header",
							name = "",
							order = 20
						},
						ignore = {
							type = "select",
							name = L["Ignored items"],
							desc = L["List of currently ignored items.  Click to remove item from ignore list."],
							get = false,
							set = "RemoveItemLink",
							values = "GetIgnoredItemLinks",
							disabled = function() return not Core:IsEnabled() end,
							order = 50
						},
						addignore = {
							type = "input",
							name = L["Add ignored item"],
							desc = L["Add new item to ignore.  Drag the item to this input box."],
							set = 	"AddNewIgnoreItem",
							disabled = function() return not Core:IsEnabled() end,
							order = 51
						},
						head3 = {
							type = "description",
							name = "\n",
							order = 52
						},
						ranks = {
							type = "select",
							name = L["Rank weights"],
							desc = L["List of member ranks in the current pool."],
							get = function() return mdkp.db.profile.editrank end,
							set = function(info, v) mdkp.db.profile.editrank = v end,
							values = "GetRankValues",
							hidden = function() return not Core:IsEnabled() or not mdkp.db.profile.useranks or not mdkp.db.profile.useranks[mdkp.db.profile.raid] end,
							order = 55
						},
						rankslider = {
							type = "range",
							name = L["Current rank weight"],
							desc = L["Higher number for higher display priority. (7 will appear above 3 on the tooltip)"],
							get = "GetRankWeight",
							set = "SetRankWeight",
							min = 1, 
							max = 10,
							step = 1,
							hidden = function() return not Core:IsEnabled() or not mdkp.db.profile.useranks or not mdkp.db.profile.useranks[mdkp.db.profile.raid] end,
							order = 60
						},
			        	},
			        },
			},
		},
		dkpmodules = {
			type = "group",
			name = L["DKP Modules"],
			desc = L["DKP Modules"],
			order = 500,
			args = {},
		},
	},
}


function MorgDKP2_Options:OnInitialize()
	if AddonLoader and AddonLoader.RemoveInterfaceOptions then
    		AddonLoader:RemoveInterfaceOptions("MorgDKP2")
	end
	LibStub("AceConfig-3.0"):RegisterOptionsTable("MorgDKP2", options)
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(MorgDKP2.db)
	options.args.profiles.order = 1000
	MorgDKP2.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MorgDKP2", "MorgDKP2")
	MorgDKP2.db.RegisterCallback(MorgDKP2, "OnProfileChanged", "OnProfileChanged")
	MorgDKP2.db.RegisterCallback(MorgDKP2, "OnProfileCopied", "OnProfileChanged")
	MorgDKP2.db.RegisterCallback(MorgDKP2, "OnProfileReset", "OnProfileChanged")
end

function MorgDKP2_Options:OnEnable()
	self:SetupModuleOptions()
	if not mdkp:GetCoreState() then mdkp:ActivateCore(true) end
end

function MorgDKP2_Options:OnProfileChanged()
	db = MorgDKP2.db.profile
	self:UpdateFrameMedia()
end

function MorgDKP2_Options:SetupModuleOptions()
	for k,v in MorgDKP2:IterateModules() do
		if v then
			local modoptions = v:GetOptions()
			if not modoptions and moduleoptions[v.moduleName] then modoptions = moduleoptions[v.moduleName] end
			local dkp = v:IsDKP()
			if modoptions then
				if dkp then 
					for prop, data in pairs(modoptions) do
						dkpmodoptions[prop] = data
					end
				else 
					options.args.modules.args[v.moduleName] = modoptions 
				end
				
			end
		end
	end
	options.args.dkpmodules.args = dkpmodoptions
end

function MorgDKP2_Options:UpdateFrameMedia()
	local LDB = mdkp:GetModule("LDBPlugin", true)
	if LDB and LDB.LDBframe then 
		self:CreateBackdrop(LDB.LDBframe)
		LDB.LDBframe:SetScale(db.media.scale)
		Core:ToggleFrameUpdate("TooltipLDB")
	end
	if Lootframe and Lootframe.lootframe then 
		self:CreateBackdrop(Lootframe.lootframe)
		self:CreateBackdrop(Lootframe.itemframe, true)
		self:CreateBackdrop(Lootframe.rollframe, true)
		self:CreateBackdrop(Lootframe.rollsummaryframe, true)
		Lootframe.font = media:Fetch("font", db.media.font)
		Lootframe.itemframe:Hide()
		Lootframe.itemframe = Lootframe:CreateItemsFrame()
		Lootframe:UpdateLootFrame(nil, nil, nil, nil, true, true)
	end
	if mdkp.frames.raidtracker then 
		local raidtracker = mdkp:GetModule("RaidTracker", true)
		self:CreateBackdrop(mdkp.frames.raidtracker, true)
		mdkp.frames.raidtracker:SetScale(db.media.scale)
		for num = 1,4 do
			if mdkp.frames.raidtracker.frame[num] then self:CreateBackdrop(mdkp.frames.raidtracker.frame[num], true, true) end
		end
		raidtracker.font = media:Fetch("font", db.media.font)
		raidtracker:RedrawFrames()
	end
	for name, data in pairs(mdkp.frames) do
		self:CreateBackdrop(data)
		data:SetScale(db.media.scale)
		Core:ToggleFrameUpdate(name)
	end
end

function MorgDKP2_Options:CreateBackdrop(f, skip, noborder)
	local temptable = f:GetBackdrop()
	local font = media:Fetch("font", db.media.font)
	if not temptable then temptable = {insets = {left = 3, right = 3, top = 3, bottom = 3},	tile = false, tileSize = 16, edgeSize = 8} end
	temptable.edgeFile = media:Fetch("border", db.media.border)
	temptable.bgFile = media:Fetch("background", db.media.background)
	f:SetBackdrop(temptable)
	f:SetBackdropColor(unpack(db.media.backdropC))
   	if not noborder then f:SetBackdropBorderColor(unpack(db.media.borderC)) 
   	else f:SetBackdropBorderColor(0,0,0,0) end
   	if f.title and not skip then 
   		f.title:SetTextColor(unpack(db.media.borderC)) 
   		f.title:SetFont(font, db.media.fontsize + 1)
   	end
   	if f.hint then
   		f.hint:SetFont(font, db.media.fontsize) 
   	end
end

function MorgDKP2_Options:SetColorOpt(info, r, g, b, a)
	local db = mdkp.db.profile.media
	local opt = info[#info]
	db[opt][1] = r
	db[opt][2] = g
	db[opt][3] = b
	db[opt][4] = a
	self:UpdateFrameMedia()
end

function MorgDKP2_Options:GetColorOpt(info)
	local db = mdkp.db.profile.media
	local opt = info[#info]
	return unpack(db[opt])
end

function MorgDKP2_Options:ClearDatabase()
	local db = mdkp.db.profile
	db.info = {}
	Core:out(L["Player database cleared!"])
end

function MorgDKP2_Options:ZeroPLayerDB()
	local db = mdkp.db.profile
	local POOL = db.raid
	for name, data in pairs(db.info) do
		data[POOL] = {points = 0, earned = 0, spent = 0}
		data.lastupdate = Core:GetServerTime()
	end
	Core:out(L["Player DKP reset to zero!"])
end

function MorgDKP2_Options:ClearRaidDB()
	if Tracker then Tracker:ClearRaidDB(true) end
end

function MorgDKP2_Options:AddNewItem(info, itemlink)
	local item = self:GetItem(itemlink)
	if item then
		Core:IteminDB(item)
		Core:out(fmt(L["Added %s to item database."], itemlink))
	end
end

function MorgDKP2_Options:AddNewIgnoreItem(info, itemlink)
	local db = mdkp.db.profile
	local item = self:GetItem(itemlink)
	if item then
		db.ignore[item] = true
		Core:out(fmt(L["Added %s to item database."], itemlink))
	end
end

function MorgDKP2_Options:GetItem(itemlink)
	if not itemlink or itemlink == "" then return end
	local _,_, item = string.find(itemlink, "item:(%d+):")
	item = tonumber(item)
	return item
end

function MorgDKP2_Options:GetRankValues()
	local db = mdkp.db.profile
	if not db.useranks or not db.useranks[db.raid] then return {"NONE"} end
	wipe(rankdata)
	local index = 1
	for rank in pairs(db.ranks[db.raid]) do
		rankdata[index] = rank
		index = index + 1
	end
	return rankdata
end

function MorgDKP2_Options:GetRankWeight()
	local db = mdkp.db.profile
	if not db.useranks or not db.useranks[db.raid] or not db.editrank then return 1 end
	return db.ranks[db.raid][rankdata[db.editrank]]
end

function MorgDKP2_Options:SetRankWeight(info, weight)
	local db = mdkp.db.profile
	db.ranks[db.raid][rankdata[db.editrank]] = weight
end

function MorgDKP2_Options:GetIgnoredItemLinks()
	wipe(ignorelinks)
	for itemid in pairs(mdkp.db.profile.ignore) do
		local _, iLink = GetItemInfo(itemid)
		table.insert(ignorelinks, iLink)
	end
	return ignorelinks
end

function MorgDKP2_Options:RemoveItemLink(info, index)
	local removelink = ignorelinks[index]
	local _,_, itemId = string.find(removelink, "item:(%d+):")
	itemId = tonumber(itemId)
	mdkp.db.profile.ignore[itemId] = nil
end

function MorgDKP2_Options:CullItemDatabase()
	local db = mdkp.db.profile
	local cull = PointsDB.db.profile.culllvl
	local culled = 0
	for item, data in pairs(db.items) do
		local _, iLink, iQuality, itemLevel, _, _, _, _, _, _ = GetItemInfo(item)
		if itemLevel and itemLevel < cull then
			if iQuality ~= 4 or (iQuality == 4 and itemLevel ~= 80 and itemLevel ~= 1) then 
				Core:debug("Removing "..iLink)
				db.items[item] = nil
				culled = culled + 1
			end
		end
	end
	if culled > 0 then
		Core:out(fmt(L["Removed %s items from the database."], culled))
	end
end

function MorgDKP2_Options:CullMemberDatabase()
	local db = mdkp.db.profile
	local cull = PointsDB.db.profile.memculllvl
	local year = date("%Y")
	local month = date("%m")
	local day = date("%d")
	local hour = date("%H")
	local min = date("%M")
	local months = floor(cull/4) 
	local days = (cull - (months * 4)) * 7
	month = month - months
	day = day - days
	if day < 1 then 
		day = day + 30
		month = month - 1
	end
	if month < 1 then 
		month = month + 12 
		year = year - 1
	end
	if strlen(day) < 2 then day = "0"..day end
	if strlen(month) < 2 then month = "0"..month end
	local calctime = year..month..day..hour..min
	local culled = 0
	for name, data in pairs(db.info) do
		if data.lastupdate < calctime then
			Core:debug("Removing "..name.." from database")
			db.info[name] = nil
			culled = culled + 1
		end
	end
	if culled > 0 then
		Core:out(fmt(L["Removed %s members from the database."], culled))
	end
end
